#!/usr/bin/perl

use Getopt::Std;

#getopts('hdlxrpsct:') || &usage;   # See the usage subroutine for more info.
#if ($opt_h) {&usage("Help Message Requested By User"); exit(0);}

###############################################################################
#
# includes
#
###############################################################################

unshift(@INC, '.');		            # local perl files

###############################################################################
#
# MAIN PROGRAM
#
###############################################################################

print "\#\#\# PREFIX \#\#\#\n";
print "Initializing...\n";

if ($^O eq "MSWin32")
{
  $onPC = 1;
}
else
{
  $onPC = 0;
}

#--- process arguments ---

if ($ARGV[0] && (-e $ARGV[0]))
{
  $fileName = $ARGV[0];
  $prefix_ctl{"mapFile"} = $fileName;
  print "...Mapping = $fileName\n";
  require("$fileName");
}
else
{
  print "ERROR: ARGV\[1\] not specified, please specify a mapping file\n";
  die;
}

if ($ARGV[1] && (-e $ARGV[1]))
{
  $fileName = $ARGV[1];
  $prefix_ctl{"inFile"} = $fileName;
  print "...Input   = $fileName\n";
}
else
{
  print "ERROR: ARGV\[1\] not specified, please specify an input file\n";
  die;
}

if ($ARGV[2])
{
  $fileName = $ARGV[2];
  $prefix_ctl{"outFile"} = $fileName;
  print "...Output  = $fileName\n";
}
else
{
  print "ERROR: ARGV\[2\] not specified, please specify an output file\n";
  die;
}

#--- process options ---



###############################################################################
#
# prefix whatever's needed
#
###############################################################################

$fileName = $prefix_ctl{"inFile"};
if (!open(IN_FILE, "<$fileName"))
{
  print "ERROR: Cannot open input file $fileName!\n"; die;
}

$fileName = $prefix_ctl{"outFile"};
if (!open(OUT_FILE, ">$fileName"))
{
  print "ERROR: Cannot open output file $fileName!\n"; die;
}

print "Prefixing...\n";
$i = 0;
$j = 0;
while (<IN_FILE>)
{
  #--- prefix all variable names -----------------------------------------------
  foreach $oldName (keys %newVariableName)
  {
    $newName = $newVariableName{$oldName};
    s/^\s*$oldName(\w*\s*=)/$newName$1/g;
    s/([+-=]\s*)$oldName/$1$newName/g;
    s/(\(\s*)$oldName(\w+\s*\))/$1$newName$2/g;
  }

  #--- prefix all model card names ---------------------------------------------
  foreach $oldName (keys %newModelCardName)
  {
    $newName = $newModelCardName{$oldName};
    s/^(\s*model\s*)$oldName/$1$newName/g;
    s/(Model\[\d+\]=\")$oldName/$1$newName/g;
    s/(=\s*)$oldName/$1$newName/g;
    s/^$oldName(\w*:)/$newName$1/g;
  }

  #--- prefix all subcircuit names ---------------------------------------------
  foreach $oldName (keys %newSubCktName)
  {
    $newName = $newSubCktName{$oldName};
    s/^(\s*define\s+)$oldName/$1$newName/g;
    s/^(\s*end\s+)$oldName/$1$newName/g;
    s/^$oldName(\w*:)/$newName$1/g;
  }

  #--- prefix all ifdef's, define's and undef's --------------------------------
  s/^(\s*\#(if|ifn|un)def\s+)/$1$ifDefPrefix/;
  s/^(\s*\#define\s+)/$1$ifDefPrefix/;

  #--- write new line ----------------------------------------------------------
  print OUT_FILE $_;

  #--- output progress information ---------------------------------------------
  $i++;
  if ($i > 20)
  {
    $i = 0;
    $j++;
    if ($j > 79)
    {
      $j = 0;
      print ".\n";
    }
    else
    {
      print ".";
    }
  }
}
close(OUT_FILE);
close(IN_FILE);
print "done!\n";
exit(0);
